/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.perseus.persistence.concurrency;

import org.objectweb.perseus.concurrency.optimistic.OptimisticConcurrencyManager;
import org.objectweb.perseus.concurrency.lib.TimeStamp;
import org.objectweb.perseus.concurrency.api.ConcurrencyException;
import org.objectweb.perseus.persistence.api.*;
import org.objectweb.perseus.cache.api.CacheEntry;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class POptimisticConcurrencyManager extends OptimisticConcurrencyManager {


    public final static String STORAGE_MANAGER_BINDING = "storage-manager";
    public final static String STATE_MANAGER_BINDING = "state-manager";

    protected StateManager stateManager;

    protected StorageManager storageManager;

    //IMPLEMENTATION OF THE UserBindingControler INTERFACE //
    //------------------------------------------------------//

    public String[] listFc() {
        return new String[] {
            STORAGE_MANAGER_BINDING,
            STATE_MANAGER_BINDING };
    }

    public Object lookupFc(String s) {
        if (STORAGE_MANAGER_BINDING.equals(s)) {
            return storageManager;
        } else if (STATE_MANAGER_BINDING.equals(s)) {
            return stateManager;
        } else {
            return super.lookupFc(s);
        }
    }

    public void bindFc(String s, Object o) {
        if (STORAGE_MANAGER_BINDING.equals(s)) {
            storageManager = (StorageManager) o;
        } else if (STATE_MANAGER_BINDING.equals(s)) {
            stateManager = (StateManager) o;
        } else super.bindFc(s, o);
    }

    public void unbindFc(String s) {
        if (STORAGE_MANAGER_BINDING.equals(s)) {
            storageManager = null;
        } else if (STATE_MANAGER_BINDING.equals(s)) {
            stateManager = null;
        } else {
            super.unbindFc(s);
        }
    }

	protected Object getState(Object ctx,
							  Object resourceId,
							  TimeStamp timeStamp,
							  Object hints) throws ConcurrencyException {
		CacheEntry ce = (CacheEntry) hints;
		WorkingSet ws = (WorkingSet) ctx;
		State state = ws.lookup(ce.getCeIdentifier());
        try {
			if (state != null)
				return state;
			state = stateManager.getReferenceState(ce);
            if (state == null)
                synchronized (ce) {
                    state = stateManager.getReferenceState(ce);
                    if (state == null) {
                        state = stateManager.createState(ce);
                        stateManager.setReferenceState(state.getCacheEntry(), state);
                        storageManager.read(ws, ce.getCeIdentifier(), state);
                        stateManager.makeClean(state);
                    }
                }
        } catch (PersistenceException e) {
            stateManager.setReferenceState(state.getCacheEntry(), null);
            stateManager.destroyState(state);
            if (e instanceof NoDSIPersistenceException) {
                //The instance does not exist in the data support, remove the lock
                if (timeStamp.close(ctx)) {
                    timeStamps.remove(ce.getCeIdentifier());
                }
            }
            throw new NoDSIConcurrencyException(e);
        }
        return stateManager.createState(state);
    }

}
