/**
 * Copyright (C) 2003-2004
 * - France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors: Olivier Lobry (olivier.lobry@rd.francetelecom.com)
 *
 */

package org.objectweb.perseus;

import org.objectweb.perseus.persistence.api.StateManager;
import org.objectweb.perseus.persistence.api.State;
import org.objectweb.perseus.cache.api.CacheEntry;
import org.objectweb.perseus.cache.api.CacheEvent;

public class StateManagerTestImpl implements StateManager {

		public StateManagerTestImpl() {

		}
	
	public State createState(CacheEntry ce) {
		if (ce.getCeObject() instanceof PString)
			return new PStringState(ce);
		if (ce.getCeObject() instanceof PInteger)
			return new PIntegerState(ce);
		return null;
	}

	public State createState(State s) {
		if (s instanceof PStringState)
			return new PStringState(s);
		if (s instanceof PIntegerState)
			return new PIntegerState(s);
		return null;
	}

	public State getReferenceState(CacheEntry ce) {
		return ((StatefullReplacebleCacheEntry) ce).getRefState();
	}

	public void setReferenceState(CacheEntry ce, State state) {
		((StatefullReplacebleCacheEntry) ce).setRefState(state);
	}

	public void destroyState(State state) {
	}

	public void makeUnexported(State state) {
		((StatusOwner) state).setStatus((byte)
				(((StatusOwner) state).getStatus() | Status.STATUS_UNEXPORTED));
	}

	public boolean isUnexported(State state) {
		return (((StatusOwner) state).getStatus() & Status.STATUS_UNEXPORTED) > 0;
	}

	public void makeExported(State state) {
		((StatusOwner) state).setStatus((byte)
				(((StatusOwner) state).getStatus() | Status.STATUS_EXPORTED));
	}

	public boolean isExported(State state) {
		return (((StatusOwner) state).getStatus() & Status.STATUS_EXPORTED) > 0;
	}

	public void makeDirty(State state) {
		byte status = ((StatusOwner) state).getStatus();
		status = (byte) (status | Status.STATUS_DIRTY);
		//status = (byte) (status | Status.STATUS_FLUSHED);
		((StatusOwner) state).setStatus(status);
	}

	public boolean isDirty(State state) {
		return (((StatusOwner) state).getStatus() & Status.STATUS_DIRTY) > 0;
	}

	public void makeClean(State state) {
		((StatusOwner) state).setStatus(Status.STATUS_CLEAN);
	}

	public void makeFlushed(State state) {
		((StatusOwner) state).setStatus((byte)
				(((StatusOwner) state).getStatus() | Status.STATUS_FLUSHED));
	}

	public boolean isFlushed(State state) {
		return (((StatusOwner) state).getStatus() & Status.STATUS_FLUSHED) > 0;
	}

	public void makeUnbound(CacheEntry ce) {
//TODO
	}

	public boolean isBound(CacheEntry ce) {
		return true;
//TODO
	}

	public void entryBound(CacheEvent event) {
//TODO
	}

	public void entryUnbound(CacheEvent event) {
//TODO
	}
}
